//  *****************************************************************************************************************
//  *                                                                                                               *
//  *                                         SpikenzieLabs.com                                                     *
//  *                                                                                                               *
//  *                                           Voice Shield                                                        *
//  *                                                                                                               *
//  *                            Program to load Audio into the Voice Shield                                        *
//  *                                                                                                               *
//  *****************************************************************************************************************
//
// BY: MARK DEMERS 
// Oct. 2008
// VERSION: 0.3
//
// DESCRIPTION:
// This is program to specifically load Audio into the Voice Shield from Spikenzielabs.com.
//
// The default serial configuration is 1200 8N1
//
// FUNCTION:
// A. Collect Audio sound bites to load into the Voice Shield
// B. Save your playlist into a TAB separated text file, [sound name with extension TAB slot number] .
// C. Connect the Arduino with Voice Shield and computer.
// D. Run this software - User selects a serial port form a list.
// E. Click on "Upload", wait for your Voice Shield is to be loaded.
// F. Use it

import ddf.minim.*;
import processing.serial.*;
import controlP5.*;

ControlP5 controlP5;

// Minim minim;        // Minim v2
// AudioPlayer song;   // Minim v2

PFont font;

Textarea myTextarea;

Textlabel tSerialPort;
Textlabel tStatus;
Textlabel tScriptFile;
Textlabel tSoundBytes;
Textlabel tBytesLenght;
Textlabel tSlotStatus;
Slider sn;
//Button bConn;

int myMainBackground = color(255,153,0);
int myAccentBackground = color(255,102,0);
int myAreaFill = 128;
int myColorBackground = color(0,0,0);
int myColorTitleText =  0xffffff00;
int WHITEtext =  0xffffffff;
int REDtext = 0x00cc0000;
int GREENtext = 0xff99ff99;

Serial port;                                                     // Create object from Serial class
AudioPlayer song;

String[] lines;
String AudioListFile = "SoundScore.txt"; // NEW SoundScore.txt // SamplesSCORE.txt
String ConnectionWarning = "Please connect to a remote Voice Shield first.";
String ArdunioError = "ERROR: Arduino did not respond, check that it is loaded with Voice Shield Sketch.";
int lineCount = 0;

int index = 0;
int temp = 0;
 // int state =3;
int i = 0;
int sampletime = 0;
String ports[] =  Serial.list();
// String STATUS      = "STOPPED";
// String SelectedPorts = "Not Connected";
int SoundByteCount = 0;
int ProgramSize = 0;
int select = 0;

int portnumber = 0;
boolean PortOpen = false;
boolean ArduinoFlag = false;
boolean ProgramFlag = false;
boolean SlotFlag = false;

int vsSlotNumber = 0;
// int SPortCount = 0;
int TabFlag = 0;                                                 // 0=Main 1=Help

float SampTime = 0;
boolean SampPlayFlag = false;

void setup()
{
  // ----------------------------------------------------------  GENERAL SET-UP
  size(552, 500);
  frameRate(25);
  background(myMainBackground);
  
 // PFont font;
    // The font must be located in the sketch's 
    // "data" directory to load successfully
  font = loadFont("ArialBoldMT17.vlw"); 
  textFont(font);

  // ----------------------------------------------------------  controlP5 SET-UP
  controlP5 = new ControlP5(this);
  controlP5.addTab(this,"Help");

  controlP5.trigger();

  controlP5.tab("default").setLabel("Main");
  controlP5.tab("default").activateEvent(true);
  controlP5.tab("default").setId(1);

  controlP5.tab("Help").activateEvent(true);
  controlP5.tab("Help").setId(2);

  //                    ("NAME",Value returned,xpos,ypos,width,Hight)
  controlP5.addButton("Program",20,30,430,80,20).setId(1);
  controlP5.addButton("Connect",4,300,430,80,20);
  
// bConn = controlP5.addButton("Con",4,360,430,80,20);

  controlP5.addButton("Stop",5,300,90,80,20);
  controlP5.addButton("Play",6,300,115,80,20);
  controlP5.addButton("Record",7,425,115,80,20).setColorLabel(color(255,0,0));

  myTextarea = controlP5.addTextarea(
  "helptext", 
  "VoiceShield Help: For interface info goto Spikenzielabs.com \n \n"+
    "1. Save your mp3 sound bytes to the same directory as this sketch. \n" +
    "2. Open a serial connection to your Arduino. \n" +
    "3. Press the program button to load the VoiceShield sounds. \n",
  50,50,width-50,height-50);
  myTextarea.setTab("Help");

  // --
 // Textlabel tTitle = controlP5.addTextlabel("Title","Voice Shield Programmer",150,10);
Textlabel tSerStatus = controlP5.addTextlabel("SerStatus","Serial Connection Status",43,100);
Textlabel tArdStatus = controlP5.addTextlabel("ArdStatus","Arduino Readiness Status",43,120);

  Textlabel tProgram = controlP5.addTextlabel("Info","Voice Shield Info",30,45);
  Textlabel tRemote = controlP5.addTextlabel("Remote","Voice Shield Remote Control",300,45);
  Textlabel tSerial = controlP5.addTextlabel("Serial","Arduino Serial Connection",300,165);
  Textlabel tFile = controlP5.addTextlabel("File","Programming Script File",30,165);

  
//  tTitle.setColorValue(myColorTitleText);
  tProgram.setColorValue(myColorTitleText);
  tRemote.setColorValue(myColorTitleText);
  tSerial.setColorValue(myColorTitleText);
  tFile.setColorValue(myColorTitleText);

   tStatus = new Textlabel(this,"Please connect Voice Shield",15,485,450,100,WHITEtext,ControlP5.synt24);
   tStatus.setFont(1);
  //  tStatus.setFont("ApleGoth17.vlw");
  
   tSlotStatus = new Textlabel(this,"N.A.",45,406,300,100,WHITEtext,ControlP5.synt24);
   tSlotStatus.setFont(1);


   tScriptFile = new Textlabel(this,AudioListFile,30,195,300,100,WHITEtext,ControlP5.synt24);
   tScriptFile.setFont(1);

   tSoundBytes = new Textlabel(this,"0 Sound Bytes",30,77,300,100,WHITEtext,ControlP5.synt24);
   tSoundBytes.setFont(1);

   tBytesLenght = new Textlabel(this,"0 Seconds each",150,77,300,100,WHITEtext,ControlP5.synt24);
   tBytesLenght.setFont(1);

   tSerialPort = new Textlabel(this,"Not Connected",300,410,300,100,WHITEtext,ControlP5.synt24);
   tSerialPort.setFont(1);
   tSerialPort.setColorValue(0x00cc0000);

  // ----------------------------------------------------------  Load Text file
  lines = loadStrings(AudioListFile);
  lineCount = lines.length;
  ScrollList l = controlP5.addScrollList("myList",30,220,200,180);

  l.setLabel("Sound Slot#  - Name");
  // *****************
  for(int i = 0; i < lines.length; i = i +1)
  {
    String[] pieces = split(lines[i], '\t');      // "Chop-up" the the play list Text file

    //NEW
    //    song = Minim.loadFile(pieces[1]);
    //    sampletime = abs((song.length()));
    //    text(sampletime, col3Start+column, (LineSize*row)+100);
    //NEW

    controlP5.Button b = l.addItem(pieces[0]+"_"+pieces[1],i);
    b.setId(100 + i);
  }

  /* will call method radio() below */
  Radio r = controlP5.addRadio("radio",300,205);
 // SPortCount = Serial.list().length;
  for(i=0; i <Serial.list().length; i=i+1)
  {
    r.add("Port ["+i+"] "+ports[i],i);
  }

  r.activate("Port ["+0+"] "+ports[0]);
  
    /* will call method slider() below */
  sn = controlP5.addSlider("Select",0,SoundByteCount,myColorBackground,300,65,170,14);


  // ----------------------------------------------------------  Minim START-UP
  Minim.start(this);                                            // always start Minim first!
//  minim = new Minim(this);   // Minim v2
  song = Minim.loadFile("/SoundBytes/"+"Monkey2.aiff");                        // Minim INIT - Play to get the Minim started
  song.play();
  delay(song.length());

}

void draw()
{
  background(myMainBackground);
  fill(myAccentBackground);
  rect(0,0,width,25);
  rect(0,height-25,width,height-25);
  
  fill(myColorTitleText);
  text("Voice Shield Programmer",190, 20);
  
  if(TabFlag ==1)
  {
    // Help Tab
  }
  else
  {
    makeroundsquare(10,15,40,100,250, myAreaFill);
    makeroundsquare(10,285,40,100,250, myAreaFill);

    makeroundsquare(10,15,160,300,250, myAreaFill);
    makeroundsquare(10,285,160,300,250, myAreaFill);
    
    stroke(0);
    line(20,60,255,60);
    line(290,60,525,60);
    line(20,180,255,180);
    line(290,180,525,180);
    
  tScriptFile.draw(this);
  tBytesLenght.draw(this);
  tStatus.draw(this);
  tSerialPort.draw(this);
  tSoundBytes.draw(this);
  tSlotStatus.draw(this);
  
  // Draw Status Dots

      ellipseMode(CORNER);
      
    if (PortOpen == true)
    {
        fill(0,255,0);
    }
    else
    {
        fill(255,0,0);
    }
      ellipse(27, 98, 10, 10);
  
      if (ArduinoFlag == true)
    {
        fill(0,255,0);
    }
    else
    {
        fill(255,0,0);
    }
      ellipse(27, 118, 10, 10);
      
      if (SlotFlag == true)
        {
          fill(0,255,0);
        }
          else
        {
          fill(255,0,0);
        }
      ellipse(30, 405, 10, 10);
  
  ellipseMode(CENTER);
  }

 // ----------------------------------------------------------------------------------------------

  if(ProgramFlag == true)
  {
//    if (index < lines.length)
//      if (index < SoundByteCount)
    if(index < ProgramSize)
    {
      if(SampPlayFlag == false)
      {
      String[] pieces = split(lines[index], '\t');      // "Chop-up" the the play list Text file
      tStatus.setValue("Record " + pieces[1] + " to address: "+ pieces[0] + ".");
      String AudFile = pieces[1];

      port.write(255);                      // Start BYTE
      port.write(3);                        // RECORD COMMAND
      port.write(int(pieces[0]));           // The ADDRESS   

      delay(950);                         // Rac time + RAC Time + Racl time 
      // N.B You may adjust this value, but without a long enough delay the start of your Audio sample may get cliped 
      // since the ISD is "slow" to start recording.

      SampPlayFlag = true;
      ISD(AudFile);
      SampTime = millis();
      SampTime = SampTime + sampletime;
    //  delay(sampletime);
      }
      else
      {
        if(millis() > SampTime)
        {
         index = index + 1;
         SampPlayFlag = false;
         
          port.write(255);                       // Start BYTE
          port.write(1);                         // STOP COMMAND
          port.write(0);                         // not a real ADDRESS (STOP does not use an Address)
         
         song.close();
        }
      }
    }
    else
    {
      index = 0;
      ProgramFlag = false;
      tStatus.setValue("Finished Recording");
    }
  }

}

void ISD(String PlayFile)
{
  song = Minim.loadFile("/SoundBytes/"+PlayFile);
  song.play();
  //  delay(song.length());
  sampletime = (song.length())+100;
 //   sampletime = (song.length());
}

/*
void mousePressed()
{
  println(mouseX+"\t"+mouseY);
}
*/

void makeroundsquare(float arcval, int xval, int yval, int hval, int wval, int fillval)
{

  // arcval is arc of the corner radius
  // xval is position on X axis
  // yval is position on Y axis
  // havl is hight
  // wval is width

  //noFill();
  fill(fillval);
  noStroke();
  rect(xval-(arcval/2),yval,wval+arcval,hval);
  rect(xval,yval-(arcval/2),wval,hval+arcval);


  stroke(0);
  strokeWeight(1.0);
  arc(xval, yval, arcval,arcval,PI, PI+PI/2); // Top Left
  arc(xval + wval, yval, arcval, arcval, PI+PI/2,0); // Top Right
  arc(xval + wval, yval+hval, arcval, arcval, 0, PI/2); // Bottom Right
  arc(xval, yval+hval, arcval, arcval, PI/2, PI); // Bottom Left
  line(xval, yval-(arcval/2), xval + wval, yval-(arcval/2)); // top
  line(xval + wval+(arcval/2), yval, xval + wval+(arcval/2), yval+hval); // Right
  line(xval + wval,  yval+hval+(arcval/2), xval, yval+hval+(arcval/2)); // Bottom
  line(xval-(arcval/2),  yval+hval, xval-(arcval/2), yval); // Left
}

void controlEvent(ControlEvent theEvent) 
{
  if(theEvent.isController()) 
  {
  //  println(theEvent.controller().id());
 //   println("some control event from "+theEvent.controller().name());
    if(theEvent.controller().id()>99)
    {
      select = (theEvent.controller().id())-100;
      vsSlotNumber = select;
      sn.setValue(vsSlotNumber);
      tStatus.setValue("Selected slot # "+vsSlotNumber);
      tStatus.setColorValue(WHITEtext);
    }
  }
  else if (theEvent.isTab()) 
  {
 //   println("tab : "+theEvent.tab().id()+" / "+theEvent.tab().name());
    if(theEvent.tab().id()==1)
    {
      TabFlag = 0;    // Delault
    }
    else
    {
      TabFlag = 1;     // Help
    }
  }

}

void Connect(float theValue) {
  if(PortOpen == false)                                                     // -- OPEN Port
  {
    tStatus.setValue("Connecting to Port ["+portnumber+"]");
   PortOpen = true;
   port = new Serial(this,ports[portnumber], 9600);

   tSerialPort.setValue("Connected to Port ["+portnumber+"] "+ports[portnumber]);
   tSerialPort.setColorValue(GREENtext);
   
    ReadSoundBytes();
  
      if(SoundByteCount <= 0)
      {
       tStatus.setValue("ERROR: Arduino did not respond, check that it is loaded with Voice Shield Sketch.");
       tStatus.setColorValue(REDtext);
       ArduinoFlag = false;
      }
      else
      {
       ArduinoFlag = true;
      }
  
  }
  else                                                                     // -- Close Port
  {
      port.clear();
      port.stop();
      
      tSerialPort.setValue("Not Connected");
      PortOpen = false;
      tSerialPort.setColorValue(REDtext);
      tStatus.setValue("Port disconnected.");
      tStatus.setColorValue(WHITEtext);
  }
}

void radio(int theValue) {
//  println("Radio Button."+ theValue);
  portnumber = theValue;
  
}

void Stop(int theValue) 
{
  // myColorBackground = theValue;
    tStatus.setValue("Remote Playback / Record: Stopped");
    tStatus.setColorValue(WHITEtext);
    
  if (PortOpen != true)
    {
    }
    else
      {
      port.write(255);                      // Start BYTE
      port.write(1);                        // STOP COMMAND
      port.write(0);                        // The ADDRESS   
      }
 // sn.setMax(350); was here just to test
}

void Program(float theValue) 
{
    if (PortOpen != true)
  {
    tStatus.setValue(ConnectionWarning);
    tStatus.setColorValue(REDtext);
  }else if (ArduinoFlag != true)
  {
    tStatus.setValue(ArdunioError);
    tStatus.setColorValue(REDtext);
  } else
  {
  tStatus.setValue("Program the ISD chip. "+theValue);
  tStatus.setColorValue(WHITEtext);
  ProgramFlag = true;
  }
}

void Record(int theValue)
{
   if (PortOpen != true)
  {
    tStatus.setValue(ConnectionWarning);
    tStatus.setColorValue(REDtext);
  }else if (ArduinoFlag != true)
  {
    tStatus.setValue(ArdunioError);
    tStatus.setColorValue(REDtext);
  } else
  {
    tStatus.setValue("RECORDING .....");
    tStatus.setColorValue(WHITEtext);
    
    
  }
}

void Play(int theValue)
{
   if (PortOpen != true)
  {
    tStatus.setValue(ConnectionWarning);
    tStatus.setColorValue(REDtext);
  }else if (ArduinoFlag != true)
  {
    tStatus.setValue(ArdunioError);
    tStatus.setColorValue(REDtext);
  } else
  {
    tStatus.setValue("PLAYING from slot #:"+vsSlotNumber);
    tStatus.setColorValue(WHITEtext);
    
      port.write(255);                      // Start BYTE
      port.write(2);                        // PLAY COMMAND
      port.write(vsSlotNumber);           // The ADDRESS   
  }
}

void Select(int theValue)                          // Used for Slider selection
{
   if (PortOpen != true)
  {
    tStatus.setValue(ConnectionWarning);
    tStatus.setColorValue(REDtext);
  }
  else
  {
    vsSlotNumber = theValue;
    if (vsSlotNumber > SoundByteCount)
    {
    tStatus.setValue("Selected slot # "+vsSlotNumber+" WARNING: SELECTED SLOT IS OUT OF RANGE. MAX IS " + SoundByteCount);
    tStatus.setColorValue(REDtext);
    }
    else
    {
    tStatus.setValue("Selected slot # "+vsSlotNumber);
    tStatus.setColorValue(WHITEtext);
    }
  }
}

void ReadSoundBytes()
{
  delay(2000);

      port.write(255);                       // Start BYTE
      port.write(4);                         // Return Sound Byte Size COMMAND
      port.write(0);                         // not a real ADDRESS (3 bytes required)
  
  delay(1000);
  if (port.available() > 0) 
  {
    String sbRX = port.readString();
  //  println(sbRX);
    SoundByteCount = int(sbRX);
    tSoundBytes.setValue(SoundByteCount+" Sound Bytes");
    temp = 240/SoundByteCount;
    tBytesLenght.setValue(temp+" Seconds per Byte");
    
    Stop(0);
    
    if(lineCount > SoundByteCount)
    {
    tStatus.setValue("WARNING: Program larger then slots in the Voice Shield, only first "+SoundByteCount+ " will be used.");
    tStatus.setColorValue(REDtext);
    
    sn.setMax(lineCount);
    SlotFlag = false;
    tSlotStatus.setValue("WARNING: Program too large.");
    tSlotStatus.setColorValue(REDtext);
    
    ProgramSize = SoundByteCount;
    
    }
   else
   {
    tStatus.setValue("Ready");
    tStatus.setColorValue(WHITEtext);
    
    tSlotStatus.setValue("OK");
    tStatus.setColorValue(WHITEtext);
    
    sn.setMax(SoundByteCount);
    SlotFlag = true;
    
    ProgramSize = lineCount;
   }
  }
}

void stop()
{
  song.close();
  Minim.stop();
  
  super.stop();
}
